;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xf31d7ac7d0f1c285ecd9821235c78776 . "./code.cg2js/cg-common-control.fasl")
(#x3fde39d07d12b799f888cbf4e8fbf790 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x227fb9e9d273fc16e05cd004b38dc1fa . "./code.cg2js/cg-yes-no-string.fasl")
(#x989c13144b3e0214dd748eea7b2860c2 . "./code.cg2js/cg-clipboard.fasl")
(#xac728f23c4060f4e7e9df433114683e6 . "./code.cg2js/cg-dde-utils.fasl")
(#xb01f675cdb8b110f3928d53d285fa104 . "./code.cg2js/cg-editable-text.fasl")
(#xe697d222d53a16633787d728545174c8 . "./code.cg2js/cg-text-or-combo.fasl")
(#x0373d77cd25775cd55d76ba76dc45fd3 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xabdcc38362f5a0e6fc016f22f5a9a792 . "./code.cg2js/cg-graphics-context.fasl")
(#xa18d88a1c8aaa77c3d865b9de2f097ce . "./code.cg2js/cg-choose-printer.fasl")
(#x829670026084fd689d9f6ee5e375f0c9 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x835266c101eb38470d0bf60db19b7264 . "./code.cg2js/cg-edit-in-place.fasl")
(#x2595ba13188d890dcf2f612637a07835 . "./code.cg2js/cg-split-bar.fasl")
(#xf32362ae2f7984ce656c754718cf4f9b . "./code.cg2js/cg-timer.fasl")
(#xe5568d45aef9fd4f308f40f70f5894b6 . "./code.cg2js/cg-progress-indicator.fasl")
(#xccd4dfd2eaab2f43c0f3301aa846c5e7 . "./code.cg2js/cg-object-editor.fasl")
(#xd4ada6dab3b6b69eca16dfb951db3b7c . "./code.cg2js/cg-fill-texture.fasl")
(#xba8174432f75507434c86181deb1a7a3 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x576762f4a03a99350f130961c37f5dcd . "./code.cg2js/cg-os-window.fasl")
(#x5280248b7008e207600414e91547a100 . "./code.cg2js/cg-color-dialog.fasl")
(#x28df2de81cce396b4aa5406edc248561 . "./code.cg2js/cg-combo-box.fasl")
(#x5f0883ed1d52ae28090abc16c95058c7 . "./code.cg2js/cg-directory-dialog.fasl")
(#xbf00cb0827726313070dc68121fd5d25 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x5de8873a06ac2e14e2c8b99fc2878e5a . "./code.cg2js/cg-tooltip.fasl")
(#xcea1c093e4f9f305a9b375f2fbb82b98 . "./code.cg2js/cg-lettered-menu.fasl")
(#x0f0b8a7f4bcc2650610cb11cdb8019fd . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x335e9612166d19d2f559d811094469fa . "./code.cg2js/cg-pixmap.fasl")
(#x456217e17d40a8fcce7f5f194256f3fe . "./code.cg2js/cg.fasl")
(#xe44ecdf345da67deb0b227efb3ef5fcd . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8d3c58e1ecb4ca3db87f610854c9976f . "./code.cg2js/cg-template-string.fasl")
(#xac4302da0f8fb9a40e51560aaf95f4c8 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe4de644f54d7c0960ee8daada81ba915 . "./code.cg2js/cg-check-box.fasl")
(#x86784846752ad0e8ec07a9f382b5a58c . "./code.cg2js/cg-toggling-widget.fasl")
(#x97681ab722a0823392ab424f73383006 . "./code.cg2js/cg-radio-button.fasl")
(#xdf2e5a01ffbc57213480e6227e8f6373 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xf2c62c4832d7470546e9a31b9ea36102 . "./code.cg2js/cg-scaling-stream.fasl")
(#x7609e5cafc64ed93c2ed5be5c83565aa . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8d799cae9201060f075da6ed92567e7d . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xc82ffc1e9eaaaa474afed1ecb747ac66 . "./code.cg2js/cg-grid-widget.fasl")
(#x0429f32f4dce2c5cce1b40f0aa4635fc . "./code.cg2js/cg-text-edit-pane.fasl")
(#xd9549fd3f7be9cabf79b4710d48fba92 . "./code.cg2js/cg-item-list.fasl")
(#x3d076e8b412d45732465c1fe4fa515c0 . "./code.cg2js/cg-paren-matching.fasl")
(#xa44bfb7c8598e37117f5ac1f70e2e176 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x6b04ee5c3dc2761e11eec29d47a43481 . "./code.cg2js/cg-html-dialog.fasl")
(#xed529403102401faecfd7bdb496aa901 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xb57da3d6cbb926c352d9ef765f899099 . "./code.cg2js/cg-project-window.fasl")
(#xe8206bea699c3237043c82a4de7eeb19 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xe4246ecf92d3cfb203ca4735566c481c . "./code.cg2js/cg-toolbar.fasl")
(#x24ec8c42a342d16c2f778baa426988b1 . "./code.cg2js/cg-caret.fasl")
(#x9485ae21fbd8c899f117e68f4688330e . "./code.cg2js/cg-scroll-bar.fasl")
(#xf32e49ac129e31bb6a9901347383b1bd . "./code.cg2js/cg-web-browser.fasl")
(#x24cbbd1212602b9ba0ac6ee0a6e78818 . "./code.cg2js/cg-dropping-outline.fasl")
(#x105b37c53dffe94bb6a7962e07285124 . "./code.cg2js/cg-font-dialog.fasl")
(#xd1f472e5030632828a11a4165472a371 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x38baa66f38637910a64d248b5e6ef267 . "./code.cg2js/cg-yes-no-list.fasl")
(#x7e2273406a40e30455f9054b1a42cc36 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x2ab6a81d41e2b476b571404465da75b1 . "./code.cg2js/cg-lamp.fasl")
(#x5a6af102bd2ce5f08b931efa8b7d9a08 . "./code.cg2js/cg-drawable.fasl")
(#xb4504f11192609157d6d41be2d55d98b . "./code.cg2js/cg-choice-list.fasl")
(#x0b743b862bf8a28b306d4b1e5a2acce8 . "./code.cg2js/cg-curve.fasl")
(#x9bba70fcc1b530dcdfff798631e30006 . "./code.cg2js/cg-base.fasl")
(#x309980f92f8e193bf7b6be4540ec6e72 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x43500322d21c3bb443e144b295571592 . "./code.cg2js/cg-printing.fasl")
(#x92288893a77f26221df493fa4f1d987c . "./code.cg2js/cg-chart-or-plot.fasl")
(#xf8fc6cef8e564987faf6565a9307680d . "./code.cg2js/cg-hotspot.fasl")
(#x38efe7e48324f25291535b65588d9dc4 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x124c7c2d1a3d0f6ed2fafab5941723cc . "./code.cg2js/cg-utility-dialog.fasl")
(#xf0c0879e35e2e0598d84d0410d952c97 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x6b5504c63bfadcd59cca4ee0319812ce . "./code.cg2js/cg-comtab.fasl")
(#xbdbc9080297f95563d9f9531a700fa3d . "./code.cg2js/cg-os-widget.fasl")
(#x99beec2a46b73899633feaf8bba7a688 . "./code.cg2js/cg-icon.fasl")
(#xde2ce9eb40a4382c008fcc687cd45e72 . "./code.cg2js/cg-dev.fasl")
(#x5bb97272c5d2be42de0d21dab08780c1 . "./code.cg2js/cg-calendar.fasl")
(#x1679e27dce1ba04cf0222aabd3f4ecdc . "./code.cg2js/cg-group-box.fasl")
(#x01ba582294a979e739aa7e6b56aaf265 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xed36535f222459acd475baf1f4cd6532 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x5acfdcb76bf1438dc69b18c56ecbeb30 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x5dc4650f72510d2e9ecd1dd77bc7f8b2 . "./code.cg2js/cg-nodes.fasl")
(#x483863a68d45346da397c61579f93b19 . "./code.cg2js/cg-intersect.fasl")
(#x982f6f4b844ea4ea96dfc68283551586 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xf6b28d2c89a3d46cede114011809da31 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x018a55be8cec5b564d548b9e27b7fec6 . "./code.cg2js/cg-class-support.fasl")
(#xf1b2f922c171b37cd0f4c9e6bac95ba7 . "./code.cg2js/cg-dialog-item.fasl")
(#xb6effadcc3c30c59d4e6717dd31f0794 . "./code.cg2js/cg-plot-widget.fasl")
(#x8a591bc22b65da6c34203e5b9ac60513 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x314d52c8d351240ba879e54847d270c4 . "./code.cg2js/cg-up-down-control.fasl")
(#x0724122a336897ff6b5b95cdc23b1c95 . "./code.cg2js/cg-rich-edit.fasl")
(#x1ac9d75643df7f6014312753a4ba0a89 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x8e30250092b961cce0f037c0112e8515 . "./code.cg2js/cg-status-bar.fasl")
(#xe68884e8f3922c8ccaddb8c951cc8932 . "./code.cg2js/cg-class-grid.fasl")
(#xf32552a2925280ef7f67ea8f3eed2690 . "./code.cg2js/cg-wrap-string.fasl")
(#xe8f8c9d2a4aed668469d008500ca4fe9 . "./code.cg2js/cg-layout.fasl")
(#xc8e2e5e12f0015e6f0783f05fdb5fc6f . "./code.cg2js/cg-lisp-widget.fasl")
(#x554259c59f831f07cb6fe82ee813a8e5 . "./code.cg2js/cg-button.fasl")
(#x85dabf1540e316818c26b5c175425b61 . "./code.cg2js/cg-string-dialog.fasl")
(#x1385feeb637c5859134fde3c1fa3e7d9 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x29818289fd42c2694a4ee34237d31463 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xec87770cda87f167b47c1bcede6d6eea . "./code.cg2js/cg-bitmap-stream.fasl")
(#x8f875f324655407c25b4afbc21b23cff . "./code.cg2js/cg-static-text.fasl")
(#x5e89073e20ef31c66b6c9f2596e4973d . "./code.cg2js/cg-outline.fasl")
(#xfb67d45253963fe88629b89b6aa138c1 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7c8028d4e25876eb5cbe6a9f21bae23e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x03f4df815045d5026fdda41700ffed47 . "./code.cg2js/cg-picture-widget.fasl")
(#x42e2c718c953a75603605debbf3922ee . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x265a18c6f2c2e40bd9f4fbbb49fd6580 . "./code.cg2js/cg-chart-widget.fasl")
(#x77f64c59102c6ce9abda6c2ad4826bdf . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xe7b186062142bca1510b4e21869a1434 . "./code.cg2js/cg-lisp-text.fasl")
(#x330cea3bdc301be0d56cebee0d83edff . "./code.cg2js/cg-palette.fasl")
(#x2fede05a3997778c9a391c1ff4e95055 . "./code.cg2js/cg-property.fasl")
(#xe1cb0f5214f2ee95eb18632482428108 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xc7c23bd5dc1843832b476fc46953114e . "./code.cg2js/cg-file-dialog.fasl")
(#x6768cc8c6f2494188bbb7d9240924547 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x1d3ad6a01d54f509d262078c2f5876c0 . "./code.cg2js/cg-get-pixmap.fasl")
(#xfe6f337549ba333626278ea7c45e6d15 . "./code.cg2js/cg-tray.fasl")
(#xb5c63af21165d3595887f2376ab16706 . "./code.cg2js/cg-acache.fasl")
(#xd60ff697d9d1d1ebe30a99cc4c32d5e7 . "./code.cg2js/cg-message-dialog.fasl")
(#x7c14e224e828d9747fe70b0cb4b55a1e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x3384b80c88784617a12908093242fde7 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x32cac15293a571c305f7eb02a6fc8b84 . "./code.cg2js/cg-tab-control.fasl")
(#x6e74fbed6f26441d5d16b9001a23204c . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x69e938b3927f8d48f61f86099dd39164 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x6e4e59e4a9de54271697e18c95156689 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x78139930c1cf8bb93a3dbab5ac500aad . "./code.cg2js/cg-menu.fasl")
(#x87b05c0ef050fed96cb2c8bd33ec1f99 . "./code.cg2js/cg-get-position.fasl")
(#xe4721aa9fd81cee0bcdd1f0320761220 . "./code.cg2js/cg-trackbar.fasl")
(#x48bb71179adbf6df00214178d122efc0 . "./code.cg2js/cg-selected-object.fasl")
(#xdc12358d6a1ea3b17e082272f227c520 . "./code.cg2js/cg-list-view.fasl")
(#x89b53a52b259a1d218dbfdad35c2d51f . "./code.cg2js/cg-html-widget.fasl")
(#x1a815cdba58f21eddc416e72f8d19cab . "./code.cg2js/cg-sample-file-menu.fasl")
(#xcc781c06ad020655069120d6eadf454e . "./code.cg2js/cg-text-widget.fasl")
(#x1c4c96fca8ce459171c5ddf5a9e0abba . "./code.idejs/ide-base.fasl")
(#x559729700d9a5c1fe78782237c274de8 . "./code.idejs/ide-process-browser.fasl")
(#x13888aed94c92d8d428aa5f431332cfc . "./code.idejs/ide-patch-dialog.fasl")
(#x69fa927a64b720a4f21fdd9639d11cf8 . "./code.idejs/ide-keyshort.fasl")
(#x52e1658f8e61d5a29d7e5ffa20a80baa . "./code.idejs/ide-window-list.fasl")
(#x8165b61eaa6d768d2c89c68cb33cca8f . "./code.idejs/ide.fasl")
(#x41488636aa79bf1ad106076eeedb093b . "./code.idejs/ide-editor.fasl")
(#x5a5ac1ead9187ac0e67e67c1043d7d2d . "./code.idejs/ide-clipboard.fasl")
(#xcbf38e6174e1d7191c0e820ca7c10e09 . "./code.idejs/ide-trace.fasl")
(#xe563a226c6be2f82cace2a270a9728a8 . "./code.idejs/ide-class-browser.fasl")
(#x532b2f4ebd2b484904fd7acd46403e0a . "./code.idejs/ide-grapher.fasl")
(#x43f79d10cd9be9749d00ddb86d175862 . "./code.idejs/ide-apropos.fasl")
(#xcfa0e57f260bc2521d46a1969a3d1292 . "./code.idejs/ide-navigator.fasl")
(#xf3cbd1b67617a3b253a064308217319c . "./code.idejs/ide-find-definitions.fasl")
(#x07063ebeb00a6e39f0965977c8487c69 . "./code.idejs/ide-project.fasl")
(#x254f481ee9a12899d78ba688de7c1b97 . "./code.idejs/ide-class-grapher.fasl")
(#xce67155b43d547fe4ae6be69c009be94 . "./code.idejs/ide-acache.fasl")
(#x00dc0401d6f9351b50dcb014b820ddda . "./code.idejs/ide-find-in-files.fasl")
(#x34ce9e18eccc25c3ae7932305676a0e9 . "./code.idejs/ide-inspector.fasl")
(#x844b1677583d503d6d2af7741716eb05 . "./code.idejs/ide-intro.fasl")
(#x05a8861efa266d0388ce80cfd3f34103 . "./code.idejs/ide-package-list.fasl")
(#x103737c755de21582d6f4ea5d5590dbb . "./code.idejs/ide-dev.fasl")
(#x81637cc3a7804250b1586f41c506ffc5 . "./code.idejs/ide-form.fasl")
(#x49a17b9a618997e974ff2e9af0f2eb06 . "./cgjs.html")
(#x44f169492307df5920f609d11cc1a34c . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
